

-- LOOT_OPENED
-- LOOT_CLOSE
-- MERCHANT_SHOW
-- MERCHANT_CLOSED

local pmt;
local copper = 0;
local gold_threshold = 5 * COPPER_PER_GOLD;
local TAX_MOD = 0.05;
local guild_id, guild, realm = nil, nil, nil;

local is_paying_tax = false;
local is_gb_open = false;
local is_mail_open = false;

-- CLASS: PayMyTax --

PayMyTax = {};

function PayMyTax:new()
	local o = {};
	self.__index = self;
	setmetatable(o, self);
	return o;
end

function PayMyTax:get_tax()
	return PAY_MY_TAX_SV[guild_id];
end

function PayMyTax:get_tax_string()
	return GetMoneyString(PAY_MY_TAX_SV[guild_id]);
end

function PayMyTax:update_tax(amount)
	PAY_MY_TAX_SV[guild_id] = PAY_MY_TAX_SV[guild_id] + amount;
end

function PayMyTax:update_guild_info()
	if IsInGuild() then
		if not guild_id then
			-- returns: guildName, guildRankName, guildRankIndex
			guild, realm = GetGuildInfo("player"), GetRealmName();

			if guild and realm then
				guild_id = format('%s-%s', guild, realm):lower();
			end
		end

        PAY_MY_TAX_SV[guild_id] = PAY_MY_TAX_SV[guild_id] or 0;

        TAX_MOD = PAY_MY_TAX_SV['pmt_tax_mod'];

        pmt:print(format('Paying %i%% Tax to: |cffffff00%s|r of %s.', 100 * TAX_MOD, guild, realm));
        self:print_tax();
	else
		guild_id = nil;
	end
end

function PayMyTax:print(msg)
	print('|cffb0c4dePay My Tax:|r ' .. msg);
end

function PayMyTax:print_tax()
	if PAY_MY_TAX_SV[guild_id] >= 1 then
		pmt:print(format('Tax: %s.', self:get_tax_string()));
	else
		pmt:print('All payed!');
	end
end

function PayMyTax:has_guild_data()
	return (guild_id ~= nil);
end

pmt = PayMyTax:new();

-- HYPERLINK HANDLER --

local function hyperlink_handler(chat, link, text, button)
	local cmd = strsub(link, 1, 4);
	if cmd == "item" then
		local _, pmt_id = strsplit(":", link);
		if pmt_id == "paymytax" then
			local tax = pmt:get_tax();
			if tax < 1 then
				pmt:print('Not enough tax to pay.');
				return;
			end

			if not is_gb_open then
				pmt:print('Please open your guild bank.');
				return;
			end

			is_paying_tax = true;
			DepositGuildBankMoney(tax);
		end
	end
end

--  EVENTS --

local pmt_frame, event_handler = pmt_ui_options_frame, {};

function tax_slider_OnHide(self)
    local value = pmt_frame.tax_slider:GetValue();
    PAY_MY_TAX_SV['pmt_tax_mod'] = floor(value + 0.5)/100;
    PayMyTax.pmt:update_guild_info();
end

function event_handler:PLAYER_LOGIN( ... )
	-- init saved variable
	if not PAY_MY_TAX_SV then
		PAY_MY_TAX_SV = {};
		PAY_MY_TAX_SV['version'] = 2;
		PAY_MY_TAX_SV['log-transactions'] = true;
        PAY_MY_TAX_SV['pmt_tax_mod'] = 0.05;
	end

    if PAY_MY_TAX_SV['version'] == 1 then
        pmt:print('Updating Saved Variables to version 2 ...');
        PAY_MY_TAX_SV['version'] = 2;
        PAY_MY_TAX_SV['pmt_tax_mod'] = 0.05;
    end

    pmt_frame.tax_slider:SetValue(PAY_MY_TAX_SV['pmt_tax_mod'] * 100);
    pmt_frame.tax_slider:SetScript("OnHide", tax_slider_OnHide);

	-- hook hyperlink functions
	hooksecurefunc("ChatFrame_OnHyperlinkShow", hyperlink_handler);
end

function event_handler:PLAYER_ENTERING_WORLD( ... )
	copper = GetMoney();
	pmt:print(format('You have %s.', GetMoneyString(copper)));
	pmt:update_guild_info();
end

function event_handler:PLAYER_MONEY( ... )
	if not pmt:has_guild_data() then
		return;
	end

	local tax, tax_mod = 0, TAX_MOD;

	if is_mail_open then
		tax_mod = 0;
	end

	local new_copper = GetMoney();
	local diff = new_copper - copper;

	copper = new_copper;

	if  diff > 0 then
		local i1 = floor(pmt:get_tax() / gold_threshold);
		tax = tax_mod * diff;
		pmt:update_tax(tax);
		local i2 = floor(pmt:get_tax() / gold_threshold);

		-- tax
		if (i2 > i1) then
			pmt:print_tax()
		end
	else
		if is_gb_open and is_paying_tax then
			pmt:update_tax(diff);
			pmt:print(format('Paid %s in tax to |cffffff00%s|r.', GetMoneyString(-diff), guild_id));
			is_paying_tax = false;
			pmt:print_tax();
			return;
		end
	end

	if PAY_MY_TAX_SV['log-transactions'] then
		local  sign = (diff < 0) and '|cffff0000-' or '|cff00ff00+';
		pmt:print(format('%s%s|r [Tax (%i%%): %s >> %s]', sign, GetMoneyString(abs(diff)), 100*TAX_MOD, GetMoneyString(tax), pmt:get_tax_string()));
	end
end

function event_handler:GUILDBANKFRAME_OPENED( ... )
	is_gb_open = true;
	pmt:print(format('Guild Tax: %s |Hitem:paymytax:create:|h[Click to Pay]|h', pmt:get_tax_string()));
end

function event_handler:GUILDBANKFRAME_CLOSED( ... )
	is_gb_open = false;
	is_paying_tax = false;
end

function event_handler:MAIL_SHOW( ... )
	is_mail_open = true;
	-- pmt:print('No tax from mail. Ignoring income.');
end

function event_handler:MAIL_CLOSED( ... )
	is_mail_open = false;
	-- pmt:print('Resuming.');
end

function event_handler:PLAYER_GUILD_UPDATE(unit)
	-- print(unit, 'Guild Update');
	if unit == 'player' then
		pmt:update_guild_info();
	end
end

-- register event callback to distribute to handlers
pmt_frame:SetScript("OnEvent", function(self, event, ...)
	-- print('PmT: Event ', event);
	event_handler[event](self, ...);
end);

for k,v in pairs(event_handler) do
	pmt_frame:RegisterEvent(k)
end

-- SLASH COMMAND --

SLASH_PAYMYTAX1, SLASH_PAYMYTAX2 = '/paymytax', '/pmt';

SlashCmdList["PAYMYTAX"] = function(message, editbox)
	local cmd = {};
	for word in message:gmatch("%S+") do
		table.insert(cmd, word);
	end

	local opIdx, opStr = next(cmd);

	if opStr == nil then
		pmt:print_tax()

	elseif opStr:lower() == "log" then
		PAY_MY_TAX_SV['log-transactions'] = not PAY_MY_TAX_SV['log-transactions'];

		if PAY_MY_TAX_SV['log-transactions'] then
			pmt:print('From now on transactions will be logged.')
		else
			pmt:print("Transactions won't be logged anymore.")
		end
    elseif opStr:lower() == "options" or opStr:lower() == "opt" then
        pmt_ui_options_frame:Show();
	end
end

PayMyTax.pmt = pmt;
